@@ -108,13 +108,6 @@ typedef enum
 	EPHY_WEB_VIEW_DOCUMENT_OTHER
 } EphyWebViewDocumentType;
 
-typedef enum
-{
-	EPHY_WEB_VIEW_ADDRESS_EXPIRE_NOW,
-	EPHY_WEB_VIEW_ADDRESS_EXPIRE_NEXT,
-	EPHY_WEB_VIEW_ADDRESS_EXPIRE_CURRENT
-} EphyWebViewAddressExpire;
-
 struct _EphyWebView
 {
   WebKitWebView parent;
@@ -177,7 +170,7 @@ void                       ephy_web_view_load_url                (EphyWebView
                                                                   const char                      *url);
 void                       ephy_web_view_copy_back_history       (EphyWebView                     *source,
                                                                   EphyWebView                     *dest);
-gboolean                   ephy_web_view_get_load_status         (EphyWebView                     *view);
+gboolean                   ephy_web_view_is_loading              (EphyWebView                     *view);
 const char *               ephy_web_view_get_loading_title       (EphyWebView                     *view);
 GdkPixbuf *                ephy_web_view_get_icon                (EphyWebView                     *view);
 EphyWebViewDocumentType    ephy_web_view_get_document_type       (EphyWebView                     *view);
@@ -195,8 +188,7 @@ void                       ephy_web_view_set_visibility          (EphyWebView
                                                                   gboolean                         visibility);
 const char *               ephy_web_view_get_typed_address       (EphyWebView                     *view);
 void                       ephy_web_view_set_typed_address       (EphyWebView                     *view,
-                                                                  const char                      *address,
-                                                                  EphyWebViewAddressExpire         expire);
+                                                                  const char                      *address);
 gboolean                   ephy_web_view_get_is_blank            (EphyWebView                     *view);
 gboolean                   ephy_web_view_has_modified_forms      (EphyWebView                     *view);
 char *                     ephy_web_view_get_location            (EphyWebView                     *view,
@@ -237,6 +229,8 @@ void                       ephy_web_view_set_loading_title       (EphyWebView
                                                                   gboolean                         is_address);
 void                       ephy_web_view_popups_manager_reset    (EphyWebView                     *view);
 
+void                       ephy_web_view_save                    (EphyWebView                     *view,
+                                                                  const char                      *uri);
 
 G_END_DECLS
 
