@@ -28,7 +28,6 @@
 #include "ephy-shell.h"
 #include "ephy-embed-persist.h"
 #include "ephy-debug.h"
-#include "ephy-command-manager.h"
 #include "window-commands.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
@@ -277,6 +276,32 @@ open_response_cb (GtkDialog *dialog, int response, EphyWindow *window)
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
+static void
+save_response_cb (GtkDialog *dialog, int response, EphyEmbed *embed)
+{
+	if (response == GTK_RESPONSE_ACCEPT)
+	{
+		char *uri, *converted;
+
+		uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+		if (uri != NULL)
+		{
+			converted = g_filename_to_utf8 (uri, -1, NULL, NULL, NULL);
+
+			if (converted != NULL)
+			{
+				EphyWebView *web_view = ephy_embed_get_web_view (embed);
+				ephy_web_view_save (web_view, converted);
+			}
+
+			g_free (converted);
+			g_free (uri);
+	        }
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
 void
 window_cmd_file_open (GtkAction *action,
 		      EphyWindow *window)
@@ -300,26 +325,21 @@ window_cmd_file_save_as (GtkAction *action,
 			 EphyWindow *window)
 {
 	EphyEmbed *embed;
-	EphyEmbedPersist *persist;
+	EphyFileChooser *dialog;
 
 	embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 	g_return_if_fail (embed != NULL);
 
-	persist = EPHY_EMBED_PERSIST
-		(g_object_new (EPHY_TYPE_EMBED_PERSIST, NULL));
+	dialog = ephy_file_chooser_new (_("Save"),
+					GTK_WIDGET (window),
+					GTK_FILE_CHOOSER_ACTION_SAVE,
+					CONF_STATE_SAVE_DIR,
+					EPHY_FILE_FILTER_ALL_SUPPORTED);
 
-	ephy_embed_persist_set_embed (persist, embed);
-	ephy_embed_persist_set_fc_title (persist, _("Save As"));
-	ephy_embed_persist_set_fc_parent (persist,GTK_WINDOW (window));
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (save_response_cb), embed);
 
-	ephy_embed_persist_set_flags
-		(persist, EPHY_EMBED_PERSIST_MAINDOC | EPHY_EMBED_PERSIST_ASK_DESTINATION);
-	ephy_embed_persist_set_persist_key
-		(persist, CONF_STATE_SAVE_DIR);
-
-	ephy_embed_persist_save (persist);
-
-	g_object_unref (G_OBJECT(persist));
+	gtk_widget_show (GTK_WIDGET (dialog));
 }
 
 void
@@ -376,8 +396,7 @@ window_cmd_edit_undo (GtkAction *action,
 
 		if (embed)
 		{
-			ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed), 
-							 "cmd_undo");
+			webkit_web_view_undo (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
 		}
 	}
 }
@@ -402,8 +421,7 @@ window_cmd_edit_redo (GtkAction *action,
 		embed = gtk_widget_get_ancestor (widget, EPHY_TYPE_EMBED);
 		if (embed)
 		{
-			ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed), 
-							 "cmd_redo");
+			webkit_web_view_redo (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
 		}
 	}
 }
@@ -423,8 +441,7 @@ window_cmd_edit_cut (GtkAction *action,
 		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
-		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
-						 "cmd_cut");
+		webkit_web_view_cut_clipboard (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
 	}
 }
 
@@ -445,8 +462,7 @@ window_cmd_edit_copy (GtkAction *action,
 		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
-		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
-						 "cmd_copy");
+		webkit_web_view_copy_clipboard (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
 	}
 }
 
@@ -467,8 +483,7 @@ window_cmd_edit_paste (GtkAction *action,
 		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
-		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
-						 "cmd_paste");
+		webkit_web_view_paste_clipboard (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
 	}
 }
 
@@ -489,8 +504,11 @@ window_cmd_edit_delete (GtkAction *action,
 		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
+		/* FIXME: TODO */
+#if 0
 		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
 						 "cmd_delete");
+#endif
 	}
 }
 
@@ -512,8 +530,7 @@ window_cmd_edit_select_all (GtkAction *action,
                   (EPHY_EMBED_CONTAINER (window));
 		g_return_if_fail (embed != NULL);
 
-		ephy_command_manager_do_command (EPHY_COMMAND_MANAGER (embed),
-						 "cmd_selectAll");
+		webkit_web_view_select_all (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed));
 	}
 }
 
